from datetime import datetime
from ftw.builder import Builder
from ftw.builder import create
from ftw.events.testing import FUNCTIONAL_TESTING
from ftw.events.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from plone import api
import transaction

events_portlet_action = '/++contextportlets++plone.rightcolumn/+/eventsarchiveportlet'


class TestEventArchivePortlets(FunctionalTestCase):
    layer = FUNCTIONAL_TESTING

    def setUp(self):
        super(TestEventArchivePortlets, self).setUp()
        self.grant('Manager')

    def _add_portlet(self, browser, context=None):
        """
        This helper method adds a events archive portlet on the given context.
        If no context is provided then the portlet will be added on the
        Plone site.
        """
        context = context or self.portal
        browser.login().visit(context, view='@@manage-portlets')
        browser.css('#portletmanager-plone-rightcolumn form')[0].fill(
            {':action': events_portlet_action}).submit()
        browser.open(context)

    @browsing
    def test_archive_portlet_available_when_there_are_events(self, browser):
        events_folder = create(Builder('event folder').titled(u'Event Folder')
                               .with_property('layout', 'event_listing'))
        create(Builder('event page').titled(u'Event Entry').within(events_folder)
               .with_property('layout', 'event_listing'))

        self._add_portlet(browser, events_folder)

        self.assertIn('Archive', browser.css('.event-archive-portlet header h2').text,
                      'Archive portlet is not here but it should be.')

    @browsing
    def test_archive_portlet_not_available_when_empty(self, browser):
        events_folder = create(Builder('event folder').titled(u'Event Folder')
                               .with_property('layout', 'event_listing'))

        self._add_portlet(browser, events_folder)

        self.assertNotIn('Archive', browser.css('dt.portletHeader').text,
                         'Archive portlet is here but it should not be.')

    @browsing
    def test_archive_portlet_summary(self, browser):
        """
        This test makes sure the summary is correct.
        """
        events_folder = create(Builder('event folder').titled(u'Event Folder')
                               .with_property('layout', 'event_listing'))
        create(Builder('event page').titled(u'Event Entry 1').within(events_folder)
               .having(start=datetime(2013, 1, 1), end=datetime(2013, 1, 1)))
        create(Builder('event page').titled(u'Event Entry 2').within(events_folder)
               .having(start=datetime(2013, 1, 11), end=datetime(2013, 1, 11)))
        create(Builder('event page').titled(u'Event Entry 3').within(events_folder)
               .having(start=datetime(2013, 2, 2), end=datetime(2013, 2, 2)))

        self._add_portlet(browser, events_folder)

        self.assertEqual(
            ['http://nohost/plone/event-folder/event_listing?archive=2013/02/01',
                'http://nohost/plone/event-folder/event_listing?archive=2013/01/01'],
            map(lambda month: month.attrib['href'], browser.css('.event-month')))

    @browsing
    def test_archive_portlet_not_available_on_plone_site(self, browser):
        """
        The events archive portlet is only rendered on events listing views.
        """
        events_folder = create(Builder('event folder').titled(u'Event Folder')
                               .with_property('layout', 'event_listing'))
        create(Builder('event page').titled(u'Event Entry 1').within(events_folder))

        self._add_portlet(browser)
        self.assertNotIn('Archive', browser.css('dt.portletHeader').text,
                         'Archive portlet is here but it should not be.')

    @browsing
    def test_archive_portlet_not_available_on_content_page(self, browser):
        """
        The events archive portlet is only rendered on events listing views.
        """
        page = create(Builder('sl content page').titled(u'Content Page'))
        events_folder = create(Builder('event folder').titled(u'Event Folder')
                               .with_property('layout', 'event_listing'))
        create(Builder('event page').titled(u'Event Entry 1').within(events_folder))

        self._add_portlet(browser, page)
        self.assertNotIn('Archive', browser.css('dt.portletHeader').text,
                         'Archive portlet is here but it should not be.')

    @browsing
    def test_archive_portlet_link(self, browser):
        """
        This test makes sure the summary is correct.
        """
        events_folder = create(Builder('event folder').titled(u'Event Folder')
                               .with_property('layout', 'event_listing'))
        create(Builder('event page').titled(u'Event Entry 1').within(events_folder)
               .having(start=datetime(2013, 1, 1), end=datetime(2013, 1, 1)))
        create(Builder('event page').titled(u'Event Entry 2').within(events_folder)
               .having(start=datetime(2013, 1, 11), end=datetime(2013, 1, 11)))

        self._add_portlet(browser, events_folder)

        browser.css('.event-month').first.click()
        self.assertEqual(2, len(browser.css('.event-item')))

    @browsing
    def test_month_with_umlaut(self, browser):
        lang_tool = api.portal.get_tool('portal_languages')
        lang_tool.setDefaultLanguage('de')
        transaction.commit()

        events_folder = create(Builder('event folder').titled(u'Event Folder')
                               .with_property('layout', 'event_listing'))
        create(Builder('event page').titled(u'Event Entry 1').within(events_folder)
               .having(start=datetime(2013, 3, 1), end=datetime(2013, 3, 1)))

        self._add_portlet(browser, events_folder)

        browser.css('.event-month').first.click()
        self.assertEqual(1, len(browser.css('.event-item')))



from setuptools import setup, find_packages
import os

version = '1.11.2'

tests_require = [
    'collective.taskqueue',
    'ftw.builder',
    'ftw.chameleon',
    'ftw.events[mopage_publisher_receiver]',
    'ftw.lawgiver',
    'ftw.testbrowser',
    'ftw.testing',
    'plone.app.testing',
    'plone.testing',
]

extras_require = {
    'tests': tests_require,
    'development': [
        'plonetheme.blueberry',
        'ftw.events[mopage_publisher_receiver]',
    ],

    # The mopage_publisher_receiver should be installed on a ftw.publisher
    # receiver installation in order to enable the mopage trigger function.
    # It should *NOT* be installed on ftw.pubsliher.sender site, since
    # the trigger will then be triggered too early.
    'mopage_publisher_receiver': [
        'collective.taskqueue',
        'ftw.publisher.receiver',
        'requests',
    ],
}

setup(
    name='ftw.events',
    version=version,
    description='Events with simplelayout.',
    long_description=open('README.rst').read() + '\n' + open(
        os.path.join('docs', 'HISTORY.txt')).read(),

    classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'Framework :: Plone :: 5.1',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    keywords='ftw events',
    author='4teamwork AG',
    author_email='mailto:info@4teamwork.ch',
    url='https://github.com/4teamwork/ftw.events',
    license='GPL2',

    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['ftw', ],
    include_package_data=True,
    zip_safe=False,

    install_requires=[
        'Plone',
        'collective.dexteritytextindexer',
        'ftw.autofeature',
        'ftw.keywordwidget',
        'ftw.profilehook',
        'ftw.referencewidget',
        'ftw.simplelayout [contenttypes] >= 1.14.0',
        'ftw.upgrade',
        'plone.api',
        'plone.app.dexterity',
        'plone.app.event [dexterity]',
        'plone.app.referenceablebehavior',
        'plone.directives.form',
        'setuptools',
    ],

    tests_require=tests_require,
    extras_require=extras_require,

    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """)



from ftw.simplelayout.interfaces import IPageConfiguration
from plone import api
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from zope.component.hooks import getSite
import transaction


def create_page_state(obj, block):
    page_state = {
        "default": [
            {
                "cols": [
                    {
                        "blocks": [
                            {
                                "uid": IUUID(block)
                            }
                        ]
                    }
                ]
            },
        ]
    }
    page_config = IPageConfiguration(obj)
    page_config.store(page_state)
    transaction.commit()


def set_allow_anonymous_view_about(state):
    site_props = api.portal.get_tool(name='portal_properties').site_properties
    site_props.allowAnonymousViewAbout = state
    transaction.commit()


def enable_behavior(behavior, portal_type):
    portal_types = getToolByName(getSite(), 'portal_types')
    behaviors = list(portal_types[portal_type].behaviors)
    behaviors.append(behavior)
    portal_types[portal_type].behaviors = tuple(behaviors)
    transaction.commit()
